@@ -26,7 +26,7 @@ def ABRPerCh(Data, Rate, TTLs, ABRCh, TimeWindow, FilterFreq, FilterOrder=4, Fil
         ABR = Analysis.SliceData(ABR, TTLs,
                                      -int(TimeWindow[0]*Rate),
                                      int(TimeWindow[1]*Rate),
-                                     AnalogTTLs=True)
+                                     )
         ABR *= 1000 # to µV
         ABRs[:,C] = ABR.mean(axis=1)
         del(ABR)
