@@ -10,6 +10,7 @@
 import numpy as np
 import os
 import scipy.signal as ssig
+import scipy.special as ssp
 from itertools import tee
 from scipy.stats.stats import pearsonr
 from scipy import fftpack
@@ -49,6 +50,7 @@ def ChDiff(Data):
 
     return(Diff)
 
+
 def Convolve(Signal1, Signal2):
     EdgeSize = Signal2.shape[0]//2
     Edge = np.zeros(EdgeSize)
@@ -68,6 +70,11 @@ def Convolve(Signal1, Signal2):
     return(Signal)
 
 
+def Coupling(H, HMax):
+    Mi = (HMax - H)/HMax
+    return(Mi)
+
+
 def CumulativeMA(Base, Add, ElNo):
     if len(Base) == 0:
         Base = Add
@@ -77,6 +84,32 @@ def CumulativeMA(Base, Add, ElNo):
     return(Base)
 
 
+def Entropy(Hist):
+    Probs = Hist/np.sum(Hist)
+    H = -np.sum(Probs[Probs > 0] * np.log(Probs[Probs > 0]))
+    HMax = np.log(Hist.shape[0])
+    return(H, HMax)
+
+
+def EucDist(A, B):
+    """
+    Function to calculate the distance between two spatial points
+
+    Parameters
+    ----------
+    A and B: list or tuple or array_like
+        Iterable containing 2 elements each such as A[0] and B[0] are the
+        x coord and A[1] and B[1] are the y coord.
+
+    Returns
+    -------
+    Dist: float or ndarray
+        Euclidean distance between points A and B.
+    """
+    Dist = ((A[1:] - A[:-1])**2 + (B[1:] - B[:-1])**2)**0.5
+    return(Dist)
+
+
 def FilterSignal(Signal, Rate, Frequency, FilterOrder=4, Coeff='butter', Type='bandpass'):
     Data = np.zeros(Signal.shape, dtype='float32')
     if len(Signal.shape) == 2:
@@ -138,6 +171,34 @@ def GenFakeTTLsRising(Rate, PulseDur, PauseBefore, PauseAfter, SampleStart, Puls
     return(FakeTTLs)
 
 
+def GetFxx(Rate, NFFT, FreqWindow):
+    """
+    Based on Chronux's getfgrid @ http://chronux.org/
+    Returns the frequencies associated with a given FFT-based computation
+
+    Parameters
+    ----------
+    Rate: int
+        Sampling rate.
+    NFFT: int
+        Nuber of FFT points.
+    FreqWindow: list or array
+        List with the lowest and highest frequency to be calculated in Hz.
+
+    Returns
+    -------
+    Fxx: array
+        Frequencies to be returned in an FFT-based computation.
+
+    """
+    Fxx = np.arange(0, Rate/NFFT, Rate)
+    Fxx = Fxx[:NFFT]
+    Freqs = np.where((Fxx >= FreqWindow[0]) * (Fxx <= FreqWindow[-1]))[0]
+    Fxx = Fxx[Freqs]
+
+    return(Fxx)
+
+
 def GetPhase(Signal):
     if len(Signal.shape) > 1:
         Data = np.zeros(Signal.shape)
@@ -205,6 +266,29 @@ def GetEventEdges(EventsDict, Ch, Proc, Rec, Edge='rise'):
 
 
 def GetNegEquiv(Number):
+    """
+    Return the negative equivalent of a percentage, for calculating differences.
+    The equivalent decrease of an increase of:
+        - 100% is -50%;
+        - 300% is -75%;
+        - 400% is -80%;
+
+    and so on. The increase can go up to infinity, but the decrease
+    can only go down to -100%. In other words, this function maps values that
+    range from 0 to inf into 0 to -100.
+
+    Parameters
+    ----------
+    Number: int or float
+        Number in the range 0-inf. If Number is < 0, it will be returned
+        unmodified.
+
+    Returns
+    -------
+    NumberNeg: int or float
+        The representation of Number into the range 0 to -100.
+
+    """
     NumberNeg = Number if Number < 0 else -(100-(100/((Number+100)/100)))
     return(NumberNeg)
 
@@ -243,8 +327,8 @@ def GetPeaks(Signal, Std=1, FixedThreshold=None):
 
 
 def GetPowerOf2(N):
-    N = int(2**np.ceil(np.log2(N)))
-    return(N)
+    Next = int(np.ceil(np.log2(N)))
+    return(Next)
 
 
 def GetRecXValues(TTLs, Rate, TimeBeforeTTL, TimeAfterTTL):
@@ -293,6 +377,16 @@ def GetTTLThreshold(TTLCh, StdNo=3):
     return(Threshold)
 
 
+def IsInt(Obj):
+    try: int(Obj); return(True)
+    except: return(False)
+
+
+def IsFloat(Obj):
+    try: float(Obj); return(True)
+    except: return(False)
+
+
 def Morlet(Freq, t, CyclesNo=5, FreqScalingFactor=1, Offset=0):
     SigmaT = CyclesNo/(2*np.pi*Freq)
 
@@ -365,6 +459,26 @@ def Pairwise(iterable):
     return zip(a, b)
 
 
+def PolygonArea(X,Y):
+    """
+    Calculate the area of a polygon based on lists of x and y coordinates.
+
+    Parameters:
+        X and Y: lists or tuples or array_like
+            Iterable containing x and y coordinates, respectively, of the
+            vertices of the polygon.
+    Returns:
+        Area: float
+            Area of the polygon defined by the vertices at the provided
+            coordinates.
+
+    Author: Madhi
+    Source: https://stackoverflow.com/a/30408825
+    """
+    Area = 0.5 * np.abs(np.dot(X, np.roll(Y,1)) - np.dot(Y, np.roll(X,1)))
+    return(Area)
+
+
 def RemapChannels(Tip, Head, Connector):
     """
     Get probe channels order. It doesn't matter what logic you follow to order
@@ -375,13 +489,12 @@ def RemapChannels(Tip, Head, Connector):
     channel map will be ordered accordingly.
 
     Example:
-        CustomAdaptor = [5, 6, 7, 8, 9, 10 ,11, 12, 13, 14, 15, 16, 1, 2, 3, 4]
-        RHAHeadstage = [16, 15, 14, 13, 12, 11, 10, 9, 1, 2, 3, 4, 5, 6, 7, 8]
+        # A16 probe connected to A16OM16 adaptor
         A16OM16 = [13, 12, 14, 11, 15, 10, 16, 9, 5, 4, 6, 3, 7, 2, 8, 1]
         A16 = {'Tip': [9, 8, 10, 7, 13, 4, 12, 5, 15, 2, 16, 1, 14, 3, 11, 6],
                'Head': [8, 7, 6, 5, 4, 3, 2, 1, 9, 10, 11, 12, 13, 14, 15, 16]}
 
-        ChannelMap = RemapChannels(A16['Tip'], A16['Head'], CustomAdaptor)
+        ChannelMap = RemapChannels(A16['Tip'], A16['Head'], A16OM16)
     """
     print('Get channel order... ', end='')
     ChNo = len(Tip)
@@ -419,6 +532,7 @@ def SubSample(Data, Rate, NewRate, t=[]):
 
 
 def UniqueStr(List, KeepOrder=False):
+    """ Better alternative: numpy.unique(List) """
     if KeepOrder:
         used = set()
         UniqueList = [x for x in List if x not in used and (used.add(x) or True)]
@@ -428,6 +542,32 @@ def UniqueStr(List, KeepOrder=False):
     return(UniqueList)
 
 
+def VonMises(Phi, PhiMean, Kappa):
+    VM = np.exp(Kappa * (np.cos(Phi - PhiMean)))
+    VM /= 2 * np.pi * ssp.iv(0, Kappa)
+    return(VM)
+
+
+def WhereMultiple(Test, Data):
+    """ Return indexes of elements in Test at Data.
+        Values in Test must be unique.
+
+        Parameters:
+            Test: array_like
+                Input array with values to be found on Data. Elements must be unique.
+            Data: array_like
+                The values against which to find each value of Test.
+
+        Returns:
+            I: ndarray, int
+                Indices of the values from Test in Data.
+
+        Taken from Divakar @ https://stackoverflow.com/a/33678576
+    """
+    I = np.nonzero(Test[:,None] == Data)[1]
+    return(I)
+
+
 ## Level 1
 def ComodulationPhaseAmp(Data, Rate, PhaseFreqBand, PhaseFreqBandWidth, PhaseFreqBandStep, AmpFreqBand, AmpFreqBandWidth, AmpFreqBandStep, FilterOrder=2, File='ComodulogramPhaseAmp', Save=False):
     """ Calculate how strongly Phase of one frequency band modulates amplitude of another. """
@@ -559,6 +699,18 @@ def GetInstFreq(Signal, Rate):
     return(InstFreq)
 
 
+def GetEntropyMI(Hist):
+    H = Entropy(Hist)[0]
+    Probs = Hist/np.sum(Hist)
+    MI = (np.log(Probs.shape[0])-H)/np.log(Probs.shape[0])
+    return(MI)
+
+
+def GetNFFT(WindowSize, Pad=0):
+    NFFT = int(2**(GetPowerOf2(WindowSize)+Pad))
+    return(NFFT)
+
+
 def GetPLVs(SignalRef, SignalResult, Rate, Freqs, FreqBand):
     if len(SignalRef.shape) > 1:
         if SignalRef.shape[1] != SignalResult.shape[1]:
@@ -599,7 +751,7 @@ def PSD(Signal, Rate, Scaling='density', WindowSize=None, NPerSeg=None, NOverlap
             F[:,C], PxxSp[:,C] = f, pxx
 
     else:
-        NFFT = GetPowerOf2(WindowSize)
+        NFFT = GetNFFT(WindowSize)
         F, PxxSp = ssig.welch(Signal, Rate, nperseg=WindowSize//2,
                                 noverlap=WindowSize//4, nfft=NFFT, detrend=False)
 
@@ -697,12 +849,30 @@ def SliceData(Signal, TTLs, NoOfSamplesBefore, NoOfSamplesAfter):
 def Spectrogram(Signal, Rate, WindowSize=None, Overlap=None, NFFT=None, Window='hann'):
     if not WindowSize: WindowSize = len(Signal)//2
     if not Overlap: Overlap = WindowSize//2
+    if not NFFT: NFFT = GetNFFT(WindowSize)
 
-    if not NFFT: NFFT = GetPowerOf2(WindowSize)
-    F, T, Sxx = ssig.spectrogram(Signal, Rate, axis=0,
-                                   nperseg=WindowSize, noverlap=Overlap,
-                                   nfft=NFFT, detrend=False, window=Window
-    )
+    F, T, Sxx = [], [], []
+
+    if len(Signal.shape) == 2:
+        for C in range(Signal.shape[1]):
+            print('Spectrogram of channel', C+1, '...')
+            f, t, sxx = Spectrogram(
+                Signal[:,C], Rate, WindowSize, Overlap, NFFT, Window
+            )
+
+            if not len(F):
+                F = f
+                T = t
+                Sxx = np.zeros((sxx.shape[0], sxx.shape[1], Signal.shape[1]), dtype='float')
+
+            Sxx[:,:,C] = sxx
+
+    else:
+        F, T, Sxx = ssig.spectrogram(
+            Signal, Rate, axis=0, nperseg=WindowSize,
+            noverlap=Overlap, nfft=NFFT, detrend=False, window=Window
+        )
 
     return(F, T, Sxx)
 
+
