@@ -36,6 +36,24 @@ def BarAutolabel(Bars, Ax, Color='k', Position='bottom'):
     return(Ax)
 
 
+def ClickCallback(Event):
+    """ Taken and modified from Joe Kington's code.
+        Source at https://stackoverflow.com/a/27566966 """
+
+    global X, Y
+    # X, Y = None, None
+    while X == Y == None:
+        print('Click on the speaker position.')
+        if Event.inaxes is not None:
+            print(Event.xdata, Event.ydata)
+            X, Y = Event.inaxes.transData.inverted().transform((Event.x, Event.y))
+        else:
+            print('Clicked coordinate is ouside axes limits')
+
+    plt.close()
+    return(None)
+
+
 def FitLegendOutside(Ax, Width=0.8, Loc='center left', Frame=False):
     # Shrink axis and fit legend outside plot
     # Taken from IanS at https://stackoverflow.com/a/4701285
@@ -59,16 +77,34 @@ def GenLegendHandle(color='r', alpha=1, label='', hatch='', marker='line', lw=2)
 
 def GetScreenInfo():
     Screen = {}
-    Screen['DPI'] = check_output(['xdpyinfo'])
-    Screen['DPI'] = str(Screen['DPI']).split('screen #')[1:]
-    Screen['DPI'] = [_.split('resolution')[1].split('\\n')[0].split('x')[0].split(' ')[-1] for _ in Screen['DPI']]
-    Screen['DPI'] = [int(_) for _ in Screen['DPI']]
+    try:
+        Screen['DPI'] = check_output(['xdpyinfo'])
+        Screen['DPI'] = str(Screen['DPI']).split('screen #')[1:]
+        Screen['DPI'] = [_.split('resolution')[1].split('\\n')[0].split('x')[0].split(' ')[-1] for _ in Screen['DPI']]
+        Screen['DPI'] = [int(_) for _ in Screen['DPI']]
+    except FileNotFoundError:
+        Screen['DPI'] = [96]
 
-    Size = check_output(['xrandr'])
-    Size = [l for l in str(Size).split('\\n') if ' connected' in l]
-    Screen['Size_mm'] = [[int(_[:-2]) for _ in f.split(' ') if 'mm' in _] for f in Size ]
-    Screen['Size_px'] = [_.split('+')[0].split('x') for f in Size for _ in f.split(' ') if 'x' in _ and '+' in _]
-    Screen['Size_px'] = [[int(_) for _ in s] for s in Screen['Size_px']]
+    try:
+        Size = check_output(['xrandr'])
+        Size = [l for l in str(Size).split('\\n') if ' connected' in l]
+        Screen['Size_mm'] = [[int(_[:-2]) for _ in f.split(' ') if 'mm' in _] for f in Size ]
+        Screen['Size_px'] = [_.split('+')[0].split('x') for f in Size for _ in f.split(' ') if 'x' in _ and '+' in _]
+        Screen['Size_px'] = [[int(_) for _ in s] for s in Screen['Size_px']]
+    except FileNotFoundError:
+        Screen['Size_mm'] = [[344, 193]]
+        Screen['Size_px'] = [[1920, 1080]]
+
+    if len(np.unique([
+        len(Screen['DPI']),
+        len(Screen['Size_mm']),
+        len(Screen['Size_px'])
+    ])) > 1:
+        # In case only one of xdpyinfo or xrandr are available, and the system
+        # has more than one screen, use only info about the first screen.
+        Screen['DPI'] = [Screen['DPI'][0]]
+        Screen['Size_mm'] = [Screen['Size_mm'][0]]
+        Screen['Size_px'] = [Screen['Size_px'][0]]
 
     ScreenNo = len(Screen['DPI'])
     Screen['Size_in'] = [[_*0.03937008 for _ in s] for s in Screen['Size_mm']]
@@ -78,6 +114,22 @@ def GetScreenInfo():
 
 
 def GetSpaces(Data, KeyList=None):
+    """
+        Calculate offsets for plotting all channels in the same plot without overlap.
+
+        Parameters:
+            Data: array_like or dict
+                If array_like, Data must be a 2D array, with dimensions (samples, channels).
+                If dict, Data must be shaped as {Channel: Samples}.
+
+            KeyList: list or None, optional
+                List of keys if Data is a dict. If None, Data keys should be int or int() convertable, and KeyList will be  a list of the sorted keys.
+                Ignored if Data is array_like.
+
+        Returns:
+            Spaces: list
+                The offset for each channel to be plot without overlap.
+    """
     if type(Data) in [list, np.ndarray, np.memmap]:
         Spaces = [min(Data[:,P[0]])-max(Data[:,P[1]]) for P in Pairwise(range(Data.shape[1]))]
 
@@ -108,11 +160,16 @@ def SameScale(Axes, Axis):
     if Axis.lower() == 'x':
         Lim = [f(Ax.get_xlim()) for f in (min,max) for Ax in Axes]
         Lim = [f(Lim) for f in (min,max)]
-        for Ax in Axes: Ax.set_xlim(Lim)
+        for Ax in Axes:
+            Ax.set_xlim(Lim)
+            Ax.spines['bottom'].set_bounds(Lim[0], Lim[1])
+
     elif Axis.lower() == 'y':
         Lim = [f(Ax.get_ylim()) for f in (min,max) for Ax in Axes]
         Lim = [f(Lim) for f in (min,max)]
-        for Ax in Axes: Ax.set_ylim(Lim)
+        for Ax in Axes:
+            Ax.set_ylim(Lim)
+            Ax.spines['left'].set_bounds(Lim[0], Lim[1])
 
     return(None)
 
@@ -130,41 +187,36 @@ def Set(Backend='TkCairo', Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=F
             'savefig.transparent' : True,
 
             'axes.titlesize'      : 10,
+            'xtick.labelsize'     : 'small',
+            'ytick.labelsize'     : 'small',
             'axes.spines.top'     : False,
             'axes.spines.right'   : False,
             'xtick.direction'     : 'out',
             'ytick.direction'     : 'out',
             'lines.linewidth'     : 1,
 
-            'figure.figsize'      : FigSize,
-            'figure.dpi'          : Screen['DPI'][0],
+            'figure.figsize'       : FigSize,
+            'figure.dpi'           : Screen['DPI'][0],
+            'figure.subplot.wspace': 0.2,
+            'figure.subplot.hspace': 0.2,
 
             'svg.fonttype'        : 'none',
             'pdf.fonttype'        : 42,
             'pgf.rcfonts'         : False,
-            'pgf.preamble'        : [],
-            # 'pgf.texsystem'       : 'pdflatex',
+            'pgf.preamble'        : '',
             'font.family'         : 'sans-serif',
             'font.serif'          : ['DejaVu Serif'],
             'font.sans-serif'     : ['DejaVu Sans'],
             'font.cursive'        : ['Zapf Chancery'],
             'font.monospace'      : ['DejaVu Sans Mono'],
-            # 'font.family'         : 'Arial',
-            # 'font.serif'          : ['Arial'],
-            # 'font.sans-serif'     : ['Arial'],
-            # 'font.cursive'        : ['Arial'],
-            # 'font.monospace'      : ['Arial'],
             'font.size'           : 10,
             'legend.fontsize'     : 8,
-            # 'text.usetex'         : True,
-            # 'text.latex.unicode'  : True,
-            # 'text.latex.preamble' : '\\usepackage{siunitx}',
         }
 
         return(Params)
 
     if Ax:
-        XLim, YLim, ZLim = None, None, None
+        XLim, YLim, ZLim, YTicks = [None]*4
 
         if 'title' in AxArgs: Ax.set_title(AxArgs['title'])
 
@@ -174,9 +226,9 @@ def Set(Backend='TkCairo', Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=F
         if 'xlim' in AxArgs: XLim = AxArgs['xlim']
 
         if 'ylabel' in AxArgs: Ax.set_ylabel(AxArgs['ylabel'])
-        if 'yticks' in AxArgs: Ax.set_yticks(AxArgs['yticks'])
         if 'yticklabels' in AxArgs: Ax.set_yticklabels(AxArgs['yticklabels'])
         if 'ylim' in AxArgs: YLim = AxArgs['ylim']
+        if 'yticks' in AxArgs: YTicks = AxArgs['yticks']
 
         if 'zlim' in AxArgs: ZLim = AxArgs['zlim']
         if ZLim:
@@ -193,41 +245,54 @@ def Set(Backend='TkCairo', Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=F
             import matplotlib.ticker as ticker
             Ax.yaxis.set_major_locator(ticker.MultipleLocator(AxArgs['ytickspacing']))
 
-        Ax.spines['bottom'].set_position(('outward', 0))
-        Ax.spines['left'].set_position(('outward', 0))
+        Ax.spines['bottom'].set_position(('outward', 2))
+        Ax.spines['left'].set_position(('outward', 2))
 
         Ax.tick_params(top=False, right=False)
         Ax.spines['right'].set_visible(False)
         Ax.spines['top'].set_visible(False)
         Ax.patch.set_visible(False)
 
-        if not XLim:
+        XTicks = Ax.get_xticks()
+        if XLim is None:
             XLim = list(Ax.get_xlim())
-#             if min(Ax.xaxis.get_data_interval()) > min(Ax.get_xticks()):
-#                 XLim[0] = Ax.get_xticks()[Ax.get_xticks()<=min(Ax.xaxis.get_data_interval())][-1]
-            if max(Ax.xaxis.get_data_interval()) < max(Ax.get_xticks()):
-                XLim[1] = Ax.get_xticks()[Ax.get_xticks()>=max(Ax.xaxis.get_data_interval())][0]
+            XInterval = Ax.xaxis.get_data_interval()
+            Tolerance = (XInterval[1]-XInterval[0])*0.1
+
+            if len(XTicks) and min(XInterval) > min(XTicks):
+                XLim[0] = XTicks[XTicks<=min(XInterval)+Tolerance][-1]
+            if len(XTicks) and max(XInterval) < max(XTicks):
+                XLim[1] = XTicks[XTicks>=max(XInterval)-Tolerance][0]
+
         Ax.set_xlim(XLim)
 
-        if len(Ax.get_xticks()):
-            XBound = [Ax.get_xticks()[Ax.get_xticks() >= XLim[0]][0],
-                      Ax.get_xticks()[Ax.get_xticks() <= XLim[1]][-1]]
+        if len(XTicks):
+            XBound = [XTicks[XTicks >= XLim[0]][0],
+                      XTicks[XTicks <= XLim[1]][-1]]
             Ax.spines['bottom'].set_bounds(XBound[0], XBound[1])
 
-        if not YLim:
+        YInterval = Ax.yaxis.get_data_interval()
+        Tolerance = (YInterval[1]-YInterval[0])*0.05
+
+        if YLim is None:# and YTicks is None:
             YLim = list(Ax.get_ylim())
-            Tolerance = (Ax.yaxis.get_data_interval()[1]-Ax.yaxis.get_data_interval()[0])*0.05
-            if min(Ax.yaxis.get_data_interval()) > min(Ax.get_yticks()):
-                YLim[0] = Ax.get_yticks()[Ax.get_yticks()<=min(Ax.yaxis.get_data_interval())+Tolerance][-1]
-            if max(Ax.yaxis.get_data_interval()) < max(Ax.get_yticks()):
-                YLim[1] = Ax.get_yticks()[Ax.get_yticks()>=max(Ax.yaxis.get_data_interval())-Tolerance][0]
-        Ax.set_ylim(YLim)
+            YTicks = Ax.get_yticks()
 
-        if len(Ax.get_yticks()):
-            YBound = [Ax.get_yticks()[Ax.get_yticks() >= YLim[0]][0],
-                      Ax.get_yticks()[Ax.get_yticks() <= YLim[1]][-1]]
+            if len(YTicks) and min(YInterval) > min(YTicks):
+                YLim[0] = YTicks[YTicks<=min(YInterval)+Tolerance][-1]
+            if len(YTicks) and max(YInterval) < max(YTicks):
+                YLim[1] = YTicks[YTicks>=max(YInterval)-Tolerance][0]
+
+        if YTicks is None: YTicks = Ax.get_yticks()
+        YTicks = np.array(YTicks)
+
+        if YTicks is not None and len(YTicks):
+            YBound = [YTicks[YTicks >= YLim[0]][0],
+                      YTicks[YTicks <= YLim[1]][-1]]
             Ax.spines['left'].set_bounds(YBound[0], YBound[1])
+            Ax.set_yticks(YTicks)
 
+        Ax.set_ylim(YLim)
 
     if Fig:
         if FigTitle: Fig.suptitle(FigTitle)
@@ -277,6 +342,17 @@ Screen = GetScreenInfo()
 FigSizeA4 = [8.27, 11.69]
 FigSize = [InchToRealSize(FigSizeA4[0]-1.97), InchToRealSize((FigSizeA4[1]-1.97)*0.3)]
 
+
+def ClickXY(Frame):
+    X, Y = None, None
+    Fig, Ax = plt.subplots()
+    Ax.imshow(Frame, vmin=0, vmax=(2**24)-1)
+    Ax.tick_params(bottom='off', top='off', labelbottom='off', right='off', left='off', labelleft='off')
+    Fig.tight_layout(pad=0)
+    Fig.canvas.callbacks.connect('button_press_event', ClickCallback)
+    plt.show()
+
+
 def Return(Function):
     if Function.lower() == 'plt':
         if 'plt' not in globals():
@@ -302,6 +378,13 @@ def Return(Function):
         else:
             return(globals()['GridSpec'])
 
+    elif Function.lower() == 'inset_axes':
+        if 'inset_axes' not in globals():
+            from mpl_toolkits.axes_grid1.inset_locator import inset_axes
+            return(inset_axes)
+        else:
+            return(globals()['inset_axes'])
+
     elif Function.lower() == 'triangulation':
         if 'Triangulation' not in globals():
             from matplotlib.tri import Triangulation
@@ -330,6 +413,13 @@ def Return(Function):
         else:
             return(globals()['Line2D'])
 
+    elif Function.lower() == 'listedcolormap':
+        if 'ListedColormap' not in globals():
+            from matplotlib.colors import ListedColormap
+            return(ListedColormap)
+        else:
+            return(globals()['ListedColormap'])
+
     elif Function.lower() == 'animation':
         if 'Animation' not in globals():
             from matplotlib.animation import FuncAnimation as Animation
@@ -354,11 +444,68 @@ def ApplyColorMapToCycle(Ax, CMap=None, Interval=None, Reverse=False):
     return(None)
 
 
+def CheckData(Data, Rate, SpaceAmpF=0.3, TimeWindow=0.2, SkipTime=10, ChLabels=[]):
+    """
+        Plot an animated visualization of Data.
+
+        Parameters:
+            Data: array_like
+                Input 2D array, with dimensions (samples, channels).
+            Rate: int
+                Data's sampling rate.
+            SpaceAmpF: float, optional
+                Amplification factor for spacing between channels. Defaults to 0.3.
+            TimeWindow: float, optional
+                Amount of Data to show  at once, in seconds. Defaults to 0.2.
+            SkipTime: int, optional
+                Amount of samples to advance per step. Defaults to 10.
+            ChLabels: list, optional
+                Label of channels. If empty, channel number will be used. Defaults to [].
+    """
+    for C in range(Data.shape[1]): Data[:,C] /= Data[:,C].max()
+    Data[:,-1] *= 0.6
+
+    Spaces = GetSpaces(Data, SpaceAmpF)
+    if not ChLabels:
+        ChLabels = list(range(1, Data.shape[1]+1))
+
+    SkipTime = int(Rate*SkipTime/1000)
+
+    try:
+        Fig, Ax = plt.subplots(dpi=150, figsize=(7,6))
+        for S in range(0,Data.shape[0]-int(TimeWindow*Rate),SkipTime):
+            t = np.arange(S, (TimeWindow*Rate)+S, 1)/Rate
+
+            YTicks = []
+            for Ch in range(Data.shape[1]):
+                Y = Data[S:int(TimeWindow*Rate)+S,Ch] - Spaces[Ch]
+                YTicks.append(Y.mean())
+                Ax.plot(t, Y, 'k')
+
+            XBar = [t[t.shape[0]//2], t[t.shape[0]//2+int(TimeWindow*0.2*Rate)]]
+            Ax.plot(XBar, [-Spaces[-1]*1.05]*2, 'k', lw=2)
+
+            Ax.set_yticks(YTicks)
+            Ax.set_yticklabels(ChLabels)
+            Ax.set_xlabel('Time [s]')
+            Ax.set_ylabel('Depth [µm]')
+
+            plt.pause(0.001)
+            Ax.clear()
+        plt.show()
+
+    except KeyboardInterrupt:
+        print('Stopping animation...')
+        plt.close('all')
+        return(None)
+
+
 def FigAx(Ax, SubPlotsArgs={}):
     ReturnAx = True
     if not Ax:
         ReturnAx = False
         plt = Return('Plt')
+
         Fig, Ax = plt.subplots(**SubPlotsArgs)
     else:
         Fig = None
@@ -385,20 +532,45 @@ def GetColors(Key):
 
 def SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show):
     if ReturnAx:
-        Set(Ax=Ax, AxArgs=AxArgs)
+        # Set(Ax=Ax, AxArgs=AxArgs)
         return(Ax)
     else:
         plt = Return('plt')
         Set(Ax=Ax, Fig=Fig, AxArgs=AxArgs)
         if Save:
             if '/' in File: os.makedirs('/'.join(File.split('/')[:-1]), exist_ok=True)
-            for E in Ext: Fig.savefig(File+'.'+E, format=E, dpi=300)
+            for E in Ext: Fig.savefig(File+'.'+E)
 
         if Show: plt.show()
         else: plt.close()
         return(None)
 
 
+def ScaleBar(
+        Fig, Ax,
+        ScalarMappableArgs={'cmap': 'inferno', 'vmin':0, 'vmax':1},
+        ColorbarArgs={}
+    ):
+    plt = Return('plt')
+
+    SM = plt.cm.ScalarMappable(
+        cmap=ScalarMappableArgs['cmap'],
+        norm=plt.Normalize(
+            vmin=ScalarMappableArgs['vmin'],
+            vmax=ScalarMappableArgs['vmax']
+        )
+    )
+    SM._A = []
+
+    Fig.colorbar(SM, ax=Ax, **ColorbarArgs)
+#     inset_axes = Return('inset_axes')
+#     Inset = inset_axes(Ax, width='5%', height='100%')
+#     Fig.colorbar(SM, cax=Inset, **ColorbarArgs)
+
+    return(None)
+
+
+
 ## Level 3
 def AllCh(Data, X=[], Colors=[], Labels=[], Leg=[], SpaceAmpF=1, ScaleBar=0, lw=2,
           Ax=None, AxArgs={}, File='AllCh', Ext=['svg'], Save=False, Show=True, SubPlotsArgs={}):
@@ -434,24 +606,21 @@ def AllCh(Data, X=[], Colors=[], Labels=[], Leg=[], SpaceAmpF=1, ScaleBar=0, lw=
     return(Result)
 
 
-def BoxPlots(Data, X=[], Colors=[], LinesAmpF=1, Width=None,
+def BoxPlots(Data, X=[], Colors=[], LinesAmpF=1, Width=None, Means=True,
              Ax=None, AxArgs={}, File='Boxplots', Ext=['svg'], Save=False, Show=True):
 
     if not len(X): X = list(range(1,len(Data)+1))
     if not len(Colors): Colors = ['k']*len(Data)
 
     Fig, Ax, ReturnAx = FigAx(Ax)
-    BoxPlot = Ax.boxplot(Data, positions=X, widths=Width, showmeans=True,
+    BoxPlot = Ax.boxplot(Data, positions=X, widths=Width, showmeans=False,
                          patch_artist=True)
 
     for I in range(len(BoxPlot['fliers'])):
         BoxPlot['fliers'][I].set_markeredgecolor(Colors[I])
 
-    for I in range(len(BoxPlot['means'])):
-        BoxPlot['means'][I].set(markeredgecolor=Colors[I], markerfacecolor=Colors[I])
-
     for I in range(len(BoxPlot['boxes'])):
-        BoxPlot['boxes'][I].set(edgecolor=Colors[I], facecolor=to_rgba(Colors[I], 0.05))
+        BoxPlot['boxes'][I].set(edgecolor=Colors[I], facecolor=to_rgba(Colors[I], 0.2))
 
     for I in range(len(BoxPlot['medians'])):
         BoxPlot['medians'][I].set(color=Colors[I])
@@ -460,6 +629,14 @@ def BoxPlots(Data, X=[], Colors=[], LinesAmpF=1, Width=None,
         for I in range(len(BoxPlot[K])):
             BoxPlot[K][I].set(color=Colors[I//2])
 
+    if Means:
+        # in matplotlib 2.2.4 the 'showmeans' feature somehow prevents
+        # inkscape from opening the resulting figure if exported to svg
+        # so means have to be manually plotted
+        Means = [np.mean(_) for _ in Data]
+        for B,Box in enumerate(Means):
+            Ax.plot(X[B], Box, color=Colors[B], marker='^')
+
     AxArgs = {**{'xticks': X}, **AxArgs}
 
     Result = SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show)
@@ -490,8 +667,34 @@ def Comodulogram(Cmdlgrm, AmpFreq, PhaseFreq, CMap='inferno',
     return(Result)
 
 
-def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread=0.2, YSpread=0, LogY=False,
-                Marker='o', Line='--', CMap=None, Paired=False,
+def Scatter3D(
+        Data, ScatterArgs={}, Elevation=30, Azimuth=-130,
+        ScaleBarArgs={},
+        Ax=None, AxArgs={}, File='Scatter3D', Ext=['pdf'], Save=False, Show=True
+    ):
+    Fig, Ax, ReturnAx = FigAx(Ax, {'subplot_kw': {'projection': '3d'}})
+
+    ScatterDefaults = dict(c=Data[:,0], cmap='inferno', edgecolors='face', alpha=1, marker='.')
+    ScatterArgs = {**ScatterDefaults, **ScatterArgs}
+
+    Ax.scatter(Data[:,0], Data[:,1], Data[:,2], **ScatterArgs)
+    Ax.view_init(Elevation, Azimuth)
+
+    if ScaleBarArgs:
+        if 'label' in ScaleBarArgs: ColorbarArgs={'label': ScaleBarArgs['label']}
+
+        ScalarMappableArgs = {'cmap': ScatterArgs['cmap'], 'vmin':0, 'vmax':1}
+        for K in ['vmin', 'vmax']:
+            if K in ScaleBarArgs: ScalarMappableArgs[K] = ScaleBarArgs[K]
+
+        ScaleBar(ScaleBarArgs['Fig'], Ax, ScalarMappableArgs, ColorbarArgs)
+
+    Result = SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show)
+    return(Result)
+
+
+def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread=0.1, YSpread=0, LogY=False,
+                Marker='o', Line='--', CMap=None, Paired=False, PlotArgs={}, ErrorArgs={},
                 Ax=None, AxArgs={}, File='ScatterMean', Ext=['svg'], Save=False, Show=True):
 
     if not len(X): X = list(range(len(Data)))
@@ -523,9 +726,17 @@ def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread
     for P,Pos in enumerate(X):
         Box, XPos, Error = Boxes[P], XPoss[P], Errors[P]
 
-        if LogY: Ax.semilogy(XPos, Box, color=Colors[P], marker=Marker, linestyle='', alpha=Alpha)
-        else: Ax.plot(XPos, Box, color=Colors[0][P], marker=Marker, linestyle='', alpha=Alpha)
-        Ax.errorbar([Pos-XSpread, Pos, Pos+XSpread], [np.mean(Box)]*3, Error, lw=3, elinewidth=1, capsize=10, color=Colors[1][P])
+        if LogY:
+            Ax.semilogy(
+                XPos, Box, color=Colors[P], markeredgecolor=Colors[0][P],
+                markerfacecolor=Colors[0][P], marker=Marker, linestyle='', alpha=Alpha
+            )
+        else:
+            Ax.plot(
+                XPos, Box, color=Colors[1][P], markeredgecolor=Colors[0][P],
+                markerfacecolor=Colors[0][P], marker=Marker, linestyle='', alpha=Alpha, **PlotArgs
+            )
+        Ax.errorbar([Pos-XSpread, Pos, Pos+XSpread], [np.mean(Box)]*3, Error, lw=3, elinewidth=1, capsize=5, color=Colors[1][P], **ErrorArgs)
 
     if Paired:
         if len(np.unique([len(_) for _ in Data])) != 1:
@@ -536,17 +747,18 @@ def ScatterMean(Data, X=[], ColorsMarkers=[], ColorsLines=[], Alpha=0.4, XSpread
             if LogY: Ax.semilogy(
                 [_[L] for _ in XPoss],
                 [_[L] for _ in Boxes],
-                color=Colors[0][P], linestyle=Line, alpha=Alpha
+                color=Colors[1][P], linestyle=Line, alpha=Alpha
             )
             else: Ax.plot(
                 [_[L] for _ in XPoss],
                 [_[L] for _ in Boxes],
-                color=Colors[0][P], linestyle=Line, alpha=Alpha
+                color=Colors[1][P], linestyle=Line, alpha=Alpha
             )
 
         if CMap: ApplyColorMapToCycle(Ax, CMap=CMap)
 
-    AxArgs = {**{'xticks': X}, **AxArgs}
+    Ax.set_xlim([X[0]-(XSpread*2), X[-1]+(XSpread*2)])
+    Ax.set_xticks(X)
 
     Result = SaveShow(ReturnAx, Fig, Ax, AxArgs, File, Ext, Save, Show)
     return(Result)
