@@ -50,7 +50,7 @@ def ISIH(ISI, ISISize=0.1, SpkNo=None,
     return(Result)
 
 
-def LinePSTH(Hist, HistX, StimWindow, StimType=[], Ax=None, AxArgs={}, File='WF-LinePSTH', Ext=['svg'], Save=False, Show=True):
+def LinePSTH(Hist, HistX, StimWindow, StimType=[], Ax=None, AxArgs={}, lw=2, File='WF-LinePSTH', Ext=['svg'], Save=False, Show=True):
     AArg = AxArgs.copy()
     Fig, Ax, ReturnAx = Plot.FigAx(Ax)
 
@@ -82,7 +82,7 @@ def LinePSTH(Hist, HistX, StimWindow, StimType=[], Ax=None, AxArgs={}, File='WF-
         BLMean = Hist[HistX<0,:].mean()
         Ax.plot([HistX[0], HistX[-1]], [BLMean]*2, 'k--')
 
-    Ax.plot(HistX, Mean, 'k', lw=2)
+    Ax.plot(HistX, Mean, 'k', lw=lw)
     Ax.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
     # Ax.set_yticklabels([str(round(float(l.get_text()),1)) for l in Ax.get_yticklabels()])
 
