@@ -50,7 +50,7 @@ def DictToList(Data):
     return(Data)
 
 
-def Read(File, ChList=[], Info={}, Verbose=True):
+def Read(File, ChList=[], Info={}, Verbose=True, AsMMap=True):
     """ Read flat interleaved binary data and return it as a numpy array. Data
         will be represented as Data[Channels, Samples].
 
@@ -71,6 +71,9 @@ def Read(File, ChList=[], Info={}, Verbose=True):
         Paths = ['/'.join(_.split('/')[:-1]) for _ in Files]
         LastCommon = os.path.commonprefix(Paths).split('/')
 
+        while LastCommon[-1] not in Paths[0].split('/'):
+            LastCommon = LastCommon[:-1]
+
         if len(LastCommon) == 1:
             LastCommon = LastCommon[-1]
         elif len(LastCommon[-1]) and LastCommon[-2] != LastCommon[-1].split('_')[0] :
@@ -99,7 +102,10 @@ def Read(File, ChList=[], Info={}, Verbose=True):
         if not Info: Info = Txt.Read(InfoFile)
 
         if os.stat(File).st_size != 0:
-            Data = np.memmap(File, dtype=Info['DType'], mode='c').reshape(Info['Shape'])
+            if AsMMap:
+                Data = np.memmap(File, dtype=Info['DType'], mode='c').reshape(Info['Shape'])
+            else:
+                Data = np.fromfile(File, dtype=Info['DType']).reshape(Info['Shape'])
 
             if ChList: Data = Data[:,[Ch-1 for Ch in ChList]]
         else:
@@ -131,6 +137,9 @@ def Write(Data, File, Info={}):
         for E,El in enumerate(Data):
             Write(El, File+'/'+File.split('/')[-1]+'_'+str(E), Info)
 
+    elif type(Data) in [int, float, str]:
+        Write(np.array(Data).reshape(1), File, Info)
+
     else:
         if '.' not in File: File +='.dat'
         if '/' in File: os.makedirs('/'.join(File.split('/')[:-1]), exist_ok=True)
