@@ -15,6 +15,9 @@ from scipy.io import wavfile
 
 def Read(File):
     Rate, Data = wavfile.read(File)
+    if len(Data.shape) == 1:
+        Data = Data.reshape((Data.shape[0], 1))
+
     return(Data, Rate)
 
 def Write(Data, Rate, File):
